#include "string.h"
#include "stdlib.h"

#include "SDL.h"
#include "SDL_net.h"
#include "SDL_ttf.h"

#include "BList.h"
#include "auxiliar.h"

#include "variables.h"
#include "mog2.h"
#include "object.h"
#include "tiles.h"
#include "mog2multiplayer.h"
#include "interfaces.h"

#include "BWindow.h"
#include "BActiveWindow.h"
#include "BStringEdDlg.h"
#include "BIntegerEdDlg.h"
#include "BChoiceDlg.h"
#include "BVChoiceDlg.h"
#include "BListWnd.h"
#include "BMultiListWnd.h"

#include "IToolsWnd.h"

#include "GamePReviewDlg.h"
#include "RMoveDlg.h"
#include "BPlaneSelectionDlg.h"

#include "editor.h"

#ifdef KITSCHY_DEBUG_MEMORY 
#include "debug_memorymanager.h"
#endif

extern int SCREEN_X;


IToolsWnd::IToolsWnd(int x,int y,int dx,int dy,EDITOR *e,TTF_Font *font) : BWindow(x,y,dx,dy,font) 
{
	BButton *b;
	editor=e;

	b=new BButton(70,4,40,14,"ADD",1);
	buttons.Add(b);

	b=new BButton(70,24,40,14,"SUB",2);
	buttons.Add(b);

	b=new BButton(4,74,70,14,"EDITING",3);
	buttons.Add(b);

	b=new BButton(4,94,70,14,"MODE",4);
	buttons.Add(b);

	b=new BButton(4,114,70,14,"GRID",5);
	buttons.Add(b);

	b=new BButton(4,140,60,14,"CLEAR",6);
	buttons.Add(b);

	b=new BButton(74,140,60,14,"COPY",7);
	buttons.Add(b);

	b=new BButton(144,140,60,14,"PASTE",8);
	buttons.Add(b);

	b=new BButton(114,4,90,14,"NEW",9);
	buttons.Add(b);

	plw=new BMultiListWnd(4,4,60,60,&plane_names,font);
	ilw=new BListWnd(114,24,90,60,&interface_names,font);

} /* IToolsWnd::IToolsWnd */ 


IToolsWnd::~IToolsWnd()
{
	delete plw;
	delete ilw;
} /* IToolsWnd::~IToolsWnd */ 


bool IToolsWnd::cycle(int stx,int sty)
{
	BWindow::cycle(stx,sty);

	/* Interface name list: */ 
	if (interface_names.EmptyP()) {
		List<MOG2_INTERFACE> l;
		MOG2_INTERFACE *i;
		char *tmp;

		interface_names.Delete();

		l.Instance(editor->game->interfaces);
		l.Rewind();
		while(l.Iterate(i)) {
			tmp=new char[strlen(i->name)+1];
			strcpy(tmp,i->name);
			interface_names.Add(tmp);
		} /* if */ 
		ilw->set_dirty();
	}

	ilw->cycle(stx+114,sty+24);
	if (ilw->is_dirty()) editor->set_all_dirty();

	if (ilw->get_selected()!=-1) {
		int i=ilw->get_selected();

		if (i>=0 && i<editor->game->interfaces.Length()) {
			editor->interf=editor->game->interfaces[i];
		} else {
			editor->interf=0;
		} /* if */ 
	} else {
		editor->interf=0;
	} /* if */ 

	{
		int right_clicked=-1;

		ilw->get_clicked_status(0,&right_clicked);

		if (right_clicked!=-1) {
			MOG2_INTERFACE *i=editor->game->interfaces[right_clicked];
			int res;
			BVChoiceDlg *dlg;
			char tmp[256];
			int x,y;

			dlg=new BVChoiceDlg(-1,-1,0,0,"Choose action:","Edit name\nDelete\nDuplicate\n",&res,font);
			dlg->Execute();
			delete dlg;

			if (res==1) {
				BStringEdDlg *dlg;

				strcpy(tmp,i->name);
				new_window_coordinates(int(SCREEN_X*0.8),60,&x,&y);
				dlg=new BStringEdDlg(x,y,int(SCREEN_X*0.8),60,"Enter interface name:",tmp,128," ",font);
				dlg->Execute();
				delete dlg;

				delete []i->name;
				i->name=new char[strlen(tmp)+1];
				strcpy(i->name,tmp);
				interface_names.Delete();
				set_dirty();
			} /* if */ 

			if (res==2) {
				if (editor->interf==i) editor->interf=0;
				editor->game->interfaces.DeleteElement(i);
				delete i;
				i=0;
				interface_names.Delete();
				set_dirty();
			} /* if */ 

			if (res==3) {
				MOG2_INTERFACE *i2,*i3;
				List<MOG2_INTERFACE> l;
				int n=0;
				char tmp[80];
				bool added=false,found;

				do{
					if (n==0) {
						sprintf(tmp,"interf copy");
					} else {
						sprintf(tmp,"interf copy_%i",n);
					} /* if */ 

					l.Instance(editor->game->interfaces);
					l.Rewind();
					found=false;
					while(!found && l.Iterate(i3)) {
						if (i3->cmp(tmp)) found=true;
					} /* while */ 

					if (!found) {
						i2=new MOG2_INTERFACE(tmp,i);
						editor->game->interfaces.Add(i2);
						added=true;
					} /* if */ 

					n++;
				}while(!added);
				interface_names.Delete();
				set_dirty();
			} /* if */ 


		} // if 
	}

	if (!editor->game->interfaces.MemberRefP(editor->interf)) editor->interf=0;

	if (editor->interf!=0) {

		while(plane_names.Length()>editor->interf->used_object_planes) {
			plw->sub_element(plane_names.Length()-1);
			plw->set_dirty();
		} /* while */ 
		while(plane_names.Length()<editor->interf->used_object_planes) {
			char tmp[80],*tmp2;
	
			sprintf(tmp,"Plane %i",plane_names.Length());
			tmp2=new char[strlen(tmp)+1];
			strcpy(tmp2,tmp);
			plw->add_element_name(tmp2);
			plw->set_selected_name(tmp2);
			
			plw->set_dirty();
		} /* while */ 

		plw->cycle(stx+4,sty+4);
		if (plw->is_dirty()) editor->set_all_dirty();

		if ((mouse_button&SDL_BUTTON(1) &&
		    !(old_mouse_button&SDL_BUTTON(1))) ||
			(mouse_button&SDL_BUTTON(3) &&
		    !(old_mouse_button&SDL_BUTTON(3)))) {

			if (editor->interf!=0 &&
				local_mouse_x>=160 &&
				local_mouse_y>=94 &&
				local_mouse_y<94+TTF_FontHeight(font) &&
				local_mouse_x<160+get_text_width("DX: 999999",font)) {
				/* Change DX: */ 
				BIntegerEdDlg *dlg;
				int y,x;

				new_window_coordinates(160,60,&x,&y);
				dlg=new BIntegerEdDlg(x,y,160,60,"Enter interface width:",&(editor->interf->dx),4,font);
				dlg->Execute();
				delete dlg;
				editor->set_all_dirty();
			} /* if */ 

			if (editor->interf!=0 &&
				local_mouse_x>=160 &&
				local_mouse_y>=104 &&
				local_mouse_y<104+TTF_FontHeight(font) &&
				local_mouse_x<160+get_text_width("DX: 999999",font)) {
				/* Change DX: */ 
				BIntegerEdDlg *dlg;
				int y,x;

				new_window_coordinates(160,60,&x,&y);
				dlg=new BIntegerEdDlg(x,y,160,60,"Enter interface height:",&(editor->interf->dy),4,font);
				dlg->Execute();
				delete dlg;
				editor->set_all_dirty();
			} /* if */ 
		} /* if */ 
	} /* if */ 

	return true;
} /* IToolsWnd::cycle */ 


void IToolsWnd::contens_size(int *dx,int *dy)
{
	int max_x=0,max_y=0;

	BWindow::contens_size(dx,dy);

	if (editor->interf!=0) {
		max_x=160+get_text_width("DX: 0000",font);
	} /* if */ 

	if (max_x<206) max_x=214;

	if (max_x>*dx) *dx=max_x;
	if (max_y>*dy) *dy=max_y;
} /* IToolsWnd::contens_size */ 


void IToolsWnd::draw_contens(SDL_Surface *sfc,int stx,int sty,int dx,int dy)
{
	SDL_FillRect(sfc,0,0);

	BWindow::draw_contens(sfc,stx,sty,dx,dy);

	draw_rectangle(sfc,2-stx,2-sty,63,63,SDL_MapRGB(sfc->format,255,255,255));

	draw_rectangle(sfc,112-stx,22-sty,93,63,SDL_MapRGB(sfc->format,255,255,255));
	ilw->draw(sfc,114-stx,24-sty,90,60);

	if (editor->iv_editmode==1) print_left("ERASE",font,255,255,255,sfc,82-stx,98-sty);
	if (editor->iv_editmode==2) print_left("REGION",font,255,255,255,sfc,82-stx,98-sty);
	if (editor->iv_editmode==3) print_left("MODIFY",font,255,255,255,sfc,82-stx,98-sty);
	if (editor->iv_editmode==4) print_left("DRAG",font,255,255,255,sfc,82-stx,98-sty);

	if (editor->iv_grid==0) print_left("1x1",font,255,255,255,sfc,82-stx,118-sty);
	if (editor->iv_grid==1) print_left("2x2",font,255,255,255,sfc,82-stx,118-sty);
	if (editor->iv_grid==2) print_left("4x4",font,255,255,255,sfc,82-stx,118-sty);
	if (editor->iv_grid==3) print_left("8x8",font,255,255,255,sfc,82-stx,118-sty);
	if (editor->iv_grid==4) print_left("16x16",font,255,255,255,sfc,82-stx,118-sty);
	if (editor->iv_grid==5) print_left("32x32",font,255,255,255,sfc,82-stx,118-sty);

	if (editor->interf!=0) {
		char tmp[80];

		plw->draw(sfc,4-stx,4-sty,60,60);

		sprintf(tmp,"DX: %i",editor->interf->dx);
		print_left(tmp,font,255,255,255,sfc,160-stx,94-sty);
		sprintf(tmp,"DY: %i",editor->interf->dy);
		print_left(tmp,font,255,255,255,sfc,160-stx,104-sty);

		sprintf(tmp,"PL %i",editor->iv_editing);
		print_left(tmp,font,255,255,255,sfc,82-stx,78-sty);		
	} /* if */ 

} /* IToolsWnd::draw_contens */ 


bool IToolsWnd::button_pressed(int ID)
{
	switch(ID) {
	case 1:
		/* Add a plane: */ 
		if (editor->interf!=0) {
			editor->interf->used_object_planes++;
			if (editor->interf->used_object_planes>MAX_OBJECT_PLANES) editor->interf->used_object_planes=MAX_OBJECT_PLANES;
			editor->set_all_dirty();
		} /* if */ 
		break;

	case 2:
		/* Sub a plane: */ 
		if (editor->interf!=0 &&
			editor->interf->used_object_planes>0) {
			editor->interf->used_object_planes--;
			editor->interf->objects[editor->interf->used_object_planes].Delete();
			editor->interf->tiles[editor->interf->used_object_planes].Delete();
			editor->set_all_dirty();
		} /* if */ 
		break;

	case 3:
		if (editor->interf!=0) {
			editor->iv_editing++;
			if (editor->iv_editing>=editor->interf->used_object_planes) editor->iv_editing=0;
			editor->set_all_dirty();
		} /* if */ 
		break;

	case 4:
		editor->iv_editmode++;
		if (editor->iv_editmode>4) editor->iv_editmode=1;
		editor->set_all_dirty();
		break;

	case 5:
		editor->iv_grid++;
		if (editor->iv_grid>=6) editor->iv_grid=0;
		editor->set_all_dirty();
		break;

	case 6:
		/* CLEAR: */ 
		if (editor->interf!=0) {

			BPlaneSelectionDlg *dlg;
			int y,x;
			int i;

			new_window_coordinates(140,100,&x,&y);
			dlg=new BPlaneSelectionDlg(x,y,140,100,editor->interf->used_object_planes,"Clear",font);
			dlg->Execute();
		
			for(i=0;i<editor->interf->used_object_planes;i++) {
				if (dlg->selected_p(i)) {
					editor->interf->objects[i].Delete();
					editor->interf->tiles[i].Delete();
				} // if 
			} // for 
			delete dlg;
			editor->set_all_dirty();

		} /* if */ 
		break;

	case 7:
		/* COPY: */ 
		if (editor->interf!=0) {

			BPlaneSelectionDlg *dlg;
			int res=0;
			int y,x;
			int i;

			new_window_coordinates(140,100,&x,&y);
			dlg=new BPlaneSelectionDlg(x,y,140,100,editor->interf->used_object_planes,"Copy",font);
			dlg->Execute();

			if (res!=0) {
				int i;
				
				for(i=0;i<MAX_OBJECT_PLANES;i++) {
				    editor->iv_clipinterf.objects[i].Delete();
				    editor->iv_clipinterf.tiles[i].Delete();
				} // for 
			} // if 

			for(i=0;i<editor->interf->used_object_planes;i++) {
				if (dlg->selected_p(i)) {
					editor->iv_clipinterf.objects[i].Copy(editor->interf->objects[i]);
				    editor->iv_clipinterf.tiles[i].Copy(editor->interf->tiles[i]);
				} else {
					editor->iv_clipinterf.objects[i].Delete();
					editor->iv_clipinterf.tiles[i].Delete();
				} // if  
			} // for  
			delete dlg;

		} /* if */ 
		break;

	case 8:
		/* PASTE: */ 
		if (editor->interf!=0) {
			BVChoiceDlg *dlg;
			int res=0;

			dlg=new BVChoiceDlg(-1,-1,0,0,"Choose action:","Paste over\nPaste replacing\n",&res,font);
			dlg->Execute();
			delete dlg;

			if (res!=0) {
				if (res==2) {
					int i;
					
					for(i=0;i<MAX_OBJECT_PLANES;i++) {
					    editor->interf->objects[i].Delete();
					    editor->interf->tiles[i].Delete();
					} /* for */ 
				} /* if */ 

				dlg=new BVChoiceDlg(-1,-1,0,0,"Choose action:","Paste single\nPaste pattern\n",&res,font);
				dlg->Execute();
				delete dlg;

				if (res==1) {
					int i;
					
					for(i=0;i<MAX_OBJECT_PLANES;i++) {
					    editor->interf->objects[i].Append(editor->iv_clipinterf.objects[i]);
					    editor->interf->tiles[i].Append(editor->iv_clipinterf.tiles[i]);
					} /* for */ 
				} /* if */ 

				if (res==2) {
					int i;
					long int stx=0,sty=0;
					long int edx=0,edy=0;
					bool first=true;
					PLACED_TILE *t;
					PLACED_OBJECT *o;

					/* Compute de bounding box of the pattern: */ 
					for(i=0;i<MAX_OBJECT_PLANES;i++) {
					    editor->iv_clipinterf.tiles[i].Rewind();
					    while(editor->iv_clipinterf.tiles[i].Iterate(t)) {
						if (first) {
							stx=t->x;
							sty=t->y;
							edx=t->x+t->tile->r.w;
							edy=t->y+t->tile->r.h;
							first=false;
						} else {
							if (t->x<stx) stx=t->x;
							if (t->y<sty) sty=t->y;
							if (t->x+t->tile->r.w>edx) edx=t->x+t->tile->r.w;
							if (t->y+t->tile->r.h>edy) edy=t->y+t->tile->r.h;
						} /* if */ 
					    } /* while */ 
					    editor->iv_clipinterf.objects[i].Rewind();
					    while(editor->iv_clipinterf.objects[i].Iterate(o)) {
						if (o->object->ntiles>0) {
							if (first) {
								stx=o->x;
								sty=o->y;
								edx=o->x+o->object->tiles[0]->r.w;
								edy=o->y+o->object->tiles[0]->r.h;
								first=false;
							} else {
								if (o->x<stx) stx=o->x;
								if (o->y<sty) sty=o->y;
								if (o->x+o->object->tiles[0]->r.w>edx) edx=o->x+o->object->tiles[0]->r.w;
								if (o->y+o->object->tiles[0]->r.h>edy) edy=o->y+o->object->tiles[0]->r.h;
							} /* if */ 
						} /* if */ 
					    } /* while */ 
					} /* for */ 
					
					edx=edx-stx;
					edy=edy-sty;

					if (edx>0 && edy>0) {
						int i;
						long int x,y;
						PLACED_TILE *t2;
						PLACED_OBJECT *o2;

						/* Repeat te pattern: */ 
						for(x=0;x<editor->interf->dx*TILESIZE;x+=edx) {
							for(y=0;y<editor->interf->dy*TILESIZE;y+=edy) {
							    for(i=0;i<MAX_OBJECT_PLANES;i++) {
								editor->iv_clipinterf.tiles[i].Rewind();
								while(editor->iv_clipinterf.tiles[i].Iterate(t)) {
									t2=new PLACED_TILE(*t);
									t2->x=(t2->x-stx)+x;
									t2->y=(t2->y-sty)+y;
									if (t2->x<editor->interf->dx*TILESIZE &&
										t2->y<editor->interf->dy*TILESIZE) editor->interf->tiles[i].Add(t2);
													    else delete t2;
								} /* while */ 
								editor->iv_clipinterf.objects[i].Rewind();
								while(editor->iv_clipinterf.objects[i].Iterate(o)) {
									o2=new PLACED_OBJECT(*o);
									o2->x=(o2->x-stx)+x;
									o2->y=(o2->y-sty)+y;
									if (o2->x<editor->interf->dx*TILESIZE &&
										o2->y<editor->interf->dy*TILESIZE) editor->interf->objects[i].Add(o2);
													    else delete o2;
								} /* while */ 
							    } /* for */ 
		    
							} /* for */ 
						} /* for */ 
					
					} /* if */ 		
				} /* if */ 
			} /* if */ 
			editor->set_all_dirty();
		}
		break;

	case 9:
		/* New interface: */ 
		{
			MOG2_INTERFACE *o;
			List<MOG2_INTERFACE> l;
			int i=0;
			char tmp[80];
			bool added=false,found;

			do{
				if (i==0) {
					sprintf(tmp,"new_interf");
				} else {
					sprintf(tmp,"new_interf_%i",i);
				} /* if */ 

				l.Instance(editor->game->interfaces);
				l.Rewind();
				found=false;
				while(!found && l.Iterate(o)) {
					if (o->cmp(tmp)) found=true;
				} /* while */ 

				if (!found) {
					o=new MOG2_INTERFACE(tmp,editor->game);
					editor->game->interfaces.Add(o);
					added=true;
				} /* if */ 

				i++;
			}while(!added);
			interface_names.Delete();
			ilw->set_selected(-1);
			set_dirty();
		}
		break;
	} /* switch */ 

	return false;
} /* IToolsWnd::button_pressed */ 
